@@ -1,6 +1,6 @@
 ##
 ## R package splines2 by Wenjie Wang and Jun Yan
-## Copyright (C) 2016-2021
+## Copyright (C) 2016-2022
 ##
 ## This file is part of the R package splines2.
 ##
@@ -20,10 +20,11 @@
 ##' Generates the basis matrix of regular M-spline, periodic M-spline, and the
 ##' corresponding integrals and derivatives.
 ##'
-##' This function contains an implementation of the close form M-spline basis
-##' based on the recursion formula given by Ramsay (1988).  For monotone
-##' regression, one can use I-splines (see \code{\link{iSpline}}) instead of
-##' M-splines.
+##' This function contains an implementation of the closed-form M-spline basis
+##' based on the recursion formula given by Ramsay (1988) or periodic M-spline
+##' basis following the procedure producing periodic B-splines given in Piegl
+##' and Tiller (1997).  For monotone regression, one can use I-splines (see
+##' \code{\link{iSpline}}) instead of M-splines.
 ##'
 ##' @inheritParams bSpline
 ##'
@@ -32,31 +33,32 @@
 ##'     M-splines, the function chooses \code{df - degree -
 ##'     as.integer(intercept)} internal knots at suitable quantiles of \code{x}
 ##'     ignoring missing values and those \code{x} outside of the boundary.  For
-##'     periodic spline based on M-spline (\code{periodic = TRUE}), \code{df -
-##'     as.integer(intercept)} internal knots will be chosen instead and the
-##'     number of internal knots must be greater or equal to the specified
-##'     \code{degree}.  If internal knots are specified via \code{knots}, the
-##'     specified \code{df} will be ignored.
+##'     periodic M-spline (\code{periodic = TRUE}), \code{df -
+##'     as.integer(intercept)} internal knots will be chosen at suitable
+##'     quantiles of \code{x} relative to the beginning of the cyclic intervals
+##'     they belong to (see Examples) and the number of internal knots must be
+##'     greater or equal to the specified \code{degree - 1}.  If internal knots
+##'     are specified via \code{knots}, the specified \code{df} will be ignored.
 ##' @param knots The internal breakpoints that define the splines.  The default
 ##'     is \code{NULL}, which results in a basis for ordinary polynomial
 ##'     regression.  Typical values are the mean or median for one knot,
 ##'     quantiles for more knots.  For periodic splines (\code{periodic =
 ##'     TRUE}), the number of knots must be greater or equal to the specified
-##'     \code{degree}.
+##'     \code{degree - 1}.
 ##' @param Boundary.knots Boundary points at which to anchor the splines.  By
 ##'     default, they are the range of \code{x} excluding \code{NA}.  If both
 ##'     \code{knots} and \code{Boundary.knots} are supplied, the basis
 ##'     parameters do not depend on \code{x}. Data can extend beyond
 ##'     \code{Boundary.knots}.  For periodic splines (\code{periodic = TRUE}),
-##'     the specified \code{Boundary.knots} define the cyclic period.
+##'     the specified boundary knots define the cyclic interval.
 ##' @param periodic A logical value.  If \code{TRUE}, the periodic splines will
 ##'     be returned instead of regular M-splines.  The default value is
 ##'     \code{FALSE}.
-##' @param derivs A non-negative integer specifying the order of derivatives of
-##'     M-splines. The default value is \code{0L} for M-spline bases.
+##' @param derivs A nonnegative integer specifying the order of derivatives of
+##'     M-splines. The default value is \code{0L} for M-spline basis functions.
 ##' @param integral A logical value.  If \code{TRUE}, the corresponding
-##'     integrals of spline bases will be returned.  The default value is
-##'     \code{FALSE}.  For periodic spline, the integral of each basis is
+##'     integrals of spline basis functions will be returned.  The default value
+##'     is \code{FALSE}.  For periodic splines, the integral of each basis is
 ##'     integrated from the left boundary knot.
 ##'
 ##' @return A numeric matrix of \code{length(x)} rows and \code{df} columns if
@@ -71,6 +73,9 @@
 ##' Ramsay, J. O. (1988). Monotone regression splines in action.
 ##' \emph{Statistical science}, 3(4), 425--441.
 ##'
+##' Piegl, L., & Tiller, W. (1997). \emph{The NURBS book}. Springer Science &
+##' Business Media.
+##'
 ##' @example inst/examples/ex-mSpline.R
 ##'
 ##' @seealso
@@ -86,7 +91,7 @@ mSpline <- function(x, df = NULL, knots = NULL, degree = 3L,
 {
     ## check inputs
     if ((derivs <- as.integer(derivs)) < 0) {
-        stop("The 'derivs' must be a non-negative integer.")
+        stop("The 'derivs' must be a nonnegative integer.")
     }
     if ((degree <- as.integer(degree)) < 0)
         stop("The 'degree' must be a nonnegative integer.")
@@ -142,7 +147,7 @@ mSpline <- function(x, df = NULL, knots = NULL, degree = 3L,
     if (! periodic && any((xx < b_knots[1L]) | (xx > b_knots[2L]))) {
         warning(wrapMessages(
             "Some 'x' values beyond boundary knots",
-            "may cause ill-conditioned bases."
+            "may cause ill-conditioned basis functions."
         ))
     }
     ## keep NA's as is
@@ -161,6 +166,6 @@ mSpline <- function(x, df = NULL, knots = NULL, degree = 3L,
         row.names(out) <- name_x
     }
     ## add class
-    class(out) <- c("matrix", "mSpline")
+    class(out) <- c("matrix", "mSpline", "splines2")
     out
 }
